@@ -1,17 +1,18 @@
-from typing import Optional, List, Dict, Any, Tuple, Callable, Iterable, Union
+from typing import Optional, List, Dict, Any, Tuple, Callable, Union, Type
 
-from StructNoSQL import PrimaryIndex, BaseField
-from StructNoSQL.models import DatabasePathElement, FieldGetter, FieldSetter, UnsafeFieldSetter, FieldRemover, FieldPathSetter
-from StructNoSQL.practical_logger import message_with_vars
+from StructNoSQL import PrimaryIndex, BaseField, TableDataModel
+from StructNoSQL.models import DatabasePathElement, FieldGetter, FieldSetter, UnsafeFieldSetter, FieldRemover, \
+    FieldPathSetter, QueryMetadata
 from StructNoSQL.tables.base_table import BaseTable
-from StructNoSQL.tables.shared_table_behaviors import _prepare_getters, _model_contain_all_index_keys
+from StructNoSQL.tables.shared_table_behaviors import _prepare_getters, _model_contain_all_index_keys, \
+    unpack_validate_retrieved_field_if_need_to, unpack_validate_multiple_retrieved_fields_if_need_to
 from StructNoSQL.utils.data_processing import navigate_into_data_with_field_path_elements
 from StructNoSQL.utils.process_render_fields_paths import process_and_make_single_rendered_database_path,\
     process_validate_data_and_make_single_rendered_database_path
 
 
 class BaseBasicTable(BaseTable):
-    def __init__(self, data_model, primary_index: PrimaryIndex):
+    def __init__(self, data_model: Type[TableDataModel], primary_index: PrimaryIndex):
         super().__init__(data_model=data_model, primary_index=primary_index)
 
     def _put_record(self, middleware: Callable[[dict], bool], record_dict_data: dict) -> bool:
@@ -82,177 +83,154 @@ class BaseBasicTable(BaseTable):
 
             return output_data
 
-    @staticmethod
-    def _process_cache_record_value(
-            data_validation: bool, value: Any, primary_key_value: Any,
-            target_field_container: Tuple[BaseField, List[DatabasePathElement]]
-    ) -> Optional[Any]:
-        if data_validation is True:
-            field_object, field_path_elements = target_field_container
-            field_object.populate(value=value)
-            validated_data, valid = field_object.validate_data()
-            return validated_data
-        else:
-            return value
-
-    @staticmethod
-    def _process_cache_record_item(
-            data_validation: bool, record_item_data: dict, primary_key_value: str,
-            target_fields_containers: Dict[str, Tuple[BaseField, List[DatabasePathElement]]]
-    ) -> dict:
-        output_data: dict = {}
-
-        if data_validation is True:
-            for item_key, item_container in target_fields_containers.items():
-                item_field_object, item_field_path_elements = item_container
-
-                matching_item_data: Optional[Any] = record_item_data.get(item_key, None)
-                item_field_object.populate(value=matching_item_data)
-                validated_data, is_valid = item_field_object.validate_data()
-                output_data[item_key] = validated_data
-        else:
-            for item_key in target_fields_containers.keys():
-                matching_item_data: Optional[Any] = record_item_data.get(item_key, None)
-                output_data[item_key] = matching_item_data
-
-        return output_data
-
     def inner_query_fields_secondary_index(
-            self, middleware: Callable[[Union[List[DatabasePathElement], Dict[str, List[DatabasePathElement]]], bool], Any],
-            target_field_container: Union[Tuple[BaseField, List[DatabasePathElement]], Dict[str, Tuple[BaseField, List[DatabasePathElement]]]],
-            is_multi_selector: bool, data_validation: bool,
-    ) -> Optional[dict]:
+            self, middleware: Callable[[List[List[DatabasePathElement]]], Tuple[Optional[List[Any]], QueryMetadata]],
+            fields_database_paths: List[List[DatabasePathElement]],
+    ) -> Tuple[Optional[dict], QueryMetadata]:
         from StructNoSQL.tables.shared_table_behaviors import _inner_query_fields_secondary_index
         return _inner_query_fields_secondary_index(
-            process_record_value=self._process_cache_record_value,
-            process_record_item=self._process_cache_record_item,
-            data_validation=data_validation,
             primary_index_name=self.primary_index_name,
             get_primary_key_database_path=self._get_primary_key_database_path,
             middleware=middleware,
-            target_field_container=target_field_container,
-            is_multi_selector=is_multi_selector
+            fields_paths_elements=fields_database_paths,
+        )
+
+    @staticmethod
+    def unpack_validate_getters_record_attributes_if_need_to(
+            data_validation: bool, record_attributes: dict,
+            single_getters_target_fields_containers: Dict[str, Tuple[BaseField, List[DatabasePathElement]]],
+            grouped_getters_target_fields_containers: Dict[str, Dict[str, Tuple[BaseField, List[DatabasePathElement]]]],
+    ):
+        def item_mutator(item_value: Any, item_field_path_elements: List[DatabasePathElement]) -> Any:
+            return item_value
+
+        from StructNoSQL.tables.shared_table_behaviors import _unpack_validate_getters_record_attributes_if_need_to
+        return _unpack_validate_getters_record_attributes_if_need_to(
+            item_mutator=item_mutator, data_validation=data_validation, record_attributes=record_attributes,
+            single_getters_target_fields_containers=single_getters_target_fields_containers,
+            grouped_getters_target_fields_containers=grouped_getters_target_fields_containers
         )
 
     def _query_field(
-            self, middleware: Callable[[Union[List[DatabasePathElement], Dict[str, List[DatabasePathElement]]], bool], List[Any]],
+            self, middleware: Callable[[List[List[DatabasePathElement]]], Tuple[Optional[List[Any]], QueryMetadata]],
             key_value: str, field_path: str, query_kwargs: Optional[dict], index_name: Optional[str], data_validation: bool
-    ) -> Optional[dict]:
+    ) -> Tuple[Optional[dict], QueryMetadata]:
 
         target_field_container, is_multi_selector = process_and_make_single_rendered_database_path(
             field_path=field_path, fields_switch=self.fields_switch, query_kwargs=query_kwargs
         )
         if index_name is not None and index_name != self.primary_index_name:
-            return self.inner_query_fields_secondary_index(
-                middleware=middleware,
-                data_validation=data_validation,
-                target_field_container=target_field_container,
-                is_multi_selector=is_multi_selector
-            )
+            output_records_values: dict = {}
+            if is_multi_selector is not True:
+                target_field_container: Tuple[BaseField, List[DatabasePathElement]]
+                fields_database_paths: List[List[DatabasePathElement]] = [target_field_container[1]]
+
+                records_attributes, query_metadata = self.inner_query_fields_secondary_index(
+                    middleware=middleware, fields_database_paths=fields_database_paths,
+                )
+                for record_key, record_item_attributes in records_attributes.items():
+                    output_records_values[record_key] = unpack_validate_retrieved_field_if_need_to(
+                        record_attributes=record_item_attributes, 
+                        target_field_container=target_field_container,
+                        data_validation=data_validation
+                    )
+            else:
+                target_field_container: Dict[str, Tuple[BaseField, List[DatabasePathElement]]]
+                fields_database_paths: List[List[DatabasePathElement]] = [item[1] for item in target_field_container.values()]
+
+                records_attributes, query_metadata = self.inner_query_fields_secondary_index(
+                    middleware=middleware, fields_database_paths=fields_database_paths,
+                )
+                for record_key, record_item_attributes in records_attributes.items():
+                    output_records_values[record_key] = unpack_validate_multiple_retrieved_fields_if_need_to(
+                        record_attributes=record_item_attributes,
+                        target_fields_containers=target_field_container,
+                        data_validation=data_validation
+                    )
+            return output_records_values, query_metadata
         else:
             # If requested index is primary index
             if is_multi_selector is not True:
                 target_field_container: Tuple[BaseField, List[DatabasePathElement]]
                 field_object, field_path_elements = target_field_container
 
-                retrieved_records_items_data: Optional[List[Any]] = middleware(field_path_elements, False)
+                retrieved_records_items_data, query_metadata = middleware([field_path_elements])
                 if retrieved_records_items_data is not None and len(retrieved_records_items_data) > 0:
                     # Since we query the primary_index, we know for a fact that we will never be returned more than
                     # one record item. Hence why we do not have a loop that iterate over the records_items_data,
                     # and that we return a dict with the only one record item being the requested key_value.
-                    return {key_value: self._process_cache_record_value(
+                    return {key_value: unpack_validate_retrieved_field_if_need_to(
                         data_validation=data_validation,
-                        value=retrieved_records_items_data[0],
-                        primary_key_value=key_value,
+                        record_attributes=retrieved_records_items_data[0],
                         target_field_container=target_field_container
-                    )}
-                return None
+                    )}, query_metadata
+                return None, query_metadata
             else:
                 target_field_container: Dict[str, Tuple[BaseField, List[DatabasePathElement]]]
 
-                if not len(target_field_container) > 0:
-                    return {key_value: {}}
-
-                fields_paths_elements: Dict[str, List[DatabasePathElement]] = {key: item[1] for key, item in target_field_container.items()}
-                retrieved_records_items_data: Optional[List[dict]] = middleware(fields_paths_elements, True)
+                fields_paths_elements: List[List[DatabasePathElement]] = [item[1] for item in target_field_container.values()]
+                retrieved_records_items_data, query_metadata = middleware(fields_paths_elements)
                 if retrieved_records_items_data is not None and len(retrieved_records_items_data) > 0:
                     # Since we query the primary_index, we know for a fact that we will never be returned more than
                     # one record item. Hence why we do not have a loop that iterate over the records_items_data,
                     # and that we return a dict with the only one record item being the requested key_value.
-                    return {key_value: self._process_cache_record_item(
+                    return {key_value: unpack_validate_multiple_retrieved_fields_if_need_to(
                         data_validation=data_validation,
-                        record_item_data=retrieved_records_items_data[0],
-                        primary_key_value=key_value,
+                        record_attributes=retrieved_records_items_data[0],
                         target_fields_containers=target_field_container
-                    )}
+                    )}, query_metadata
+                return None, query_metadata
 
     def _query_multiple_fields(
-            self, middleware: Callable[[Dict[str, List[DatabasePathElement]], bool], List[Any]],
+            self, middleware: Callable[[List[List[DatabasePathElement]]], Tuple[Optional[List[Any]], QueryMetadata]],
             key_value: str, getters: Dict[str, FieldGetter], index_name: Optional[str], data_validation: bool
-    ):
-        getters_database_paths, single_getters_target_fields_containers, grouped_getters_database_paths_elements = (
+    ) -> Tuple[Optional[dict], QueryMetadata]:
+        getters_database_paths, single_getters_target_fields_containers, grouped_getters_target_fields_containers = (
             _prepare_getters(fields_switch=self.fields_switch, getters=getters)
         )
-        if len(grouped_getters_database_paths_elements) > 0:
-            raise Exception(f"grouped_getters_database_paths_elements not yet supported")
 
         if index_name is None or index_name == self.primary_index_name:
-            single_getters_database_paths_elements: Dict[str, List[DatabasePathElement]] = (
-                {key: item[1] for key, item in single_getters_target_fields_containers.items()}
-            )
-            retrieved_records_items_data: Optional[List[Any]] = middleware(single_getters_database_paths_elements, True)
+            retrieved_records_items_data, query_metadata = middleware(getters_database_paths)
             if retrieved_records_items_data is not None and len(retrieved_records_items_data) > 0:
                 # Since we query the primary_index, we know for a fact that we will never be returned more than
                 # one record item. Hence why we do not have a loop that iterate over the records_items_data,
                 # and that we return a dict with the only one record item being the requested key_value.
-                return {key_value: self._process_cache_record_item(
-                    data_validation=data_validation,
-                    record_item_data=retrieved_records_items_data[0],
-                    primary_key_value=key_value,
-                    target_fields_containers=single_getters_target_fields_containers
-                )}
-            return None
+                record_item_value: Dict[str, Any] = self.unpack_validate_getters_record_attributes_if_need_to(
+                    data_validation=data_validation, record_attributes=retrieved_records_items_data[0],
+                    single_getters_target_fields_containers=single_getters_target_fields_containers,
+                    grouped_getters_target_fields_containers=grouped_getters_target_fields_containers
+                )
+                return {key_value: record_item_value}, query_metadata
+            return None, query_metadata
         else:
-            return self.inner_query_fields_secondary_index(
-                middleware=middleware,
-                data_validation=data_validation,
-                target_field_container=single_getters_target_fields_containers,
-                is_multi_selector=True
+            records_attributes, query_metadata = self.inner_query_fields_secondary_index(
+                middleware=middleware, fields_database_paths=getters_database_paths,
             )
-
-    @staticmethod
-    def _unpack_getters_response_item(
-            data_validation: bool, response_item: dict,
-            single_getters_database_paths_elements: Dict[str, Tuple[BaseField, List[DatabasePathElement]]],
-            grouped_getters_database_paths_elements: Dict[str, Dict[str, Tuple[BaseField, List[DatabasePathElement]]]],
-    ):
-        def item_mutator(item: Any):
-            return item
-
-        from StructNoSQL.tables.shared_table_behaviors import _base_unpack_getters_response_item_v2
-        return _base_unpack_getters_response_item_v2(
-            item_mutator=item_mutator, data_validation=data_validation, response_item=response_item,
-            single_getters_database_paths_elements=single_getters_database_paths_elements,
-            grouped_getters_database_paths_elements=grouped_getters_database_paths_elements
-        )
+            output_records_values: dict = {
+                record_key: self.unpack_validate_getters_record_attributes_if_need_to(
+                    data_validation=data_validation, record_attributes=record_item_attributes,
+                    single_getters_target_fields_containers=single_getters_target_fields_containers,
+                    grouped_getters_target_fields_containers=grouped_getters_target_fields_containers
+                ) for record_key, record_item_attributes in records_attributes.items()
+            }
+            return output_records_values, query_metadata
 
     def _get_multiple_fields(
             self, middleware: Callable[[List[List[DatabasePathElement]]], Any],
             getters: Dict[str, FieldGetter], data_validation: bool
     ) -> Dict[str, Optional[Any]]:
 
-        getters_database_paths, single_getters_database_paths_elements, grouped_getters_database_paths_elements = (
+        getters_database_paths, single_getters_target_fields_containers, grouped_getters_target_fields_containers = (
             _prepare_getters(fields_switch=self.fields_switch, getters=getters)
         )
-        response_data: Optional[dict] = middleware(getters_database_paths)
-        if response_data is None:
+        record_attributes: Optional[dict] = middleware(getters_database_paths)
+        if record_attributes is None:
             return {getter_key: None for getter_key in getters.keys()}
 
-        return self._unpack_getters_response_item(
-            response_item=response_data,
-            single_getters_database_paths_elements=single_getters_database_paths_elements,
-            grouped_getters_database_paths_elements=grouped_getters_database_paths_elements,
-            data_validation=data_validation
+        return self.unpack_validate_getters_record_attributes_if_need_to(
+            data_validation=data_validation, record_attributes=record_attributes,
+            single_getters_target_fields_containers=single_getters_target_fields_containers,
+            grouped_getters_target_fields_containers=grouped_getters_target_fields_containers,
         )
 
     def _update_field(
@@ -467,14 +445,14 @@ class BaseBasicTable(BaseTable):
                 grouped_removers_field_paths_elements[remover_key] = target_field_container
                 removers_database_paths.extend(fields_paths_elements)
 
-        response_attributes: Optional[Any] = middleware(removers_database_paths)
-        if response_attributes is None:
+        record_attributes: Optional[dict] = middleware(removers_database_paths)
+        if record_attributes is None:
             return None
 
-        return self._unpack_getters_response_item(
-            data_validation=data_validation, response_item=response_attributes,
-            single_getters_database_paths_elements=removers_field_paths_elements,
-            grouped_getters_database_paths_elements=grouped_removers_field_paths_elements
+        return self.unpack_validate_getters_record_attributes_if_need_to(
+            data_validation=data_validation, record_attributes=record_attributes,
+            single_getters_target_fields_containers=removers_field_paths_elements,
+            grouped_getters_target_fields_containers=grouped_removers_field_paths_elements
         )
 
     def _grouped_delete_multiple_fields(
