@@ -3,17 +3,30 @@ from typing import Optional, Tuple, List, Any
 from StructNoSQL.models import DatabasePathElement
 from StructNoSQL.fields import BaseItem, BaseField, MapModel
 from StructNoSQL.practical_logger import message_with_vars
-from StructNoSQL.utils.decimals import float_to_decimal, float_to_decimal_serializer
 from StructNoSQL.utils.misc_fields_items import try_to_get_primitive_default_type_of_item
 
 
-def validate_data(value: Any, expected_value_type: Any, item_type_to_return_to: Optional[BaseItem] = None) -> Tuple[Any, bool]:
+def _types_match(type_to_check: type, expected_type: type) -> bool:
+    processed_expected_type: type = try_to_get_primitive_default_type_of_item(expected_type)
+    # The type_to_check will always be a primitive Python type, where as the expected_type can also be a StructNoSQL
+    # model type. In which case, we try_to_get_primitive_default_type_of_item. If the type was not a StructNoSQL
+    # model type and did not had a primitive_default_type, the returned type will be the source type we passed.
+    if processed_expected_type is Any:
+        return True
+    elif type_to_check != processed_expected_type:
+        return False
+    return True
+
+def validate_data(
+        value: Any, expected_value_type: Any,
+        item_type_to_return_to: Optional[BaseItem] = None
+) -> Tuple[Any, bool]:
     value_type = type(value)
     # We do not try_to_get_primitive_default_type_of_item here, because depending on the value_type, we might trigger different behaviors.
     # For example, a list or tuple will be considered as collection of multiple fields types that needs to be looked at individually.
 
     if expected_value_type == Any:
-        return float_to_decimal_serializer(value), True
+        return value, True
 
     if type(expected_value_type) in [list, tuple]:
         has_found_match = False
@@ -61,7 +74,7 @@ def validate_data(value: Any, expected_value_type: Any, item_type_to_return_to:
                     if item_matching_validation_model_variable is not None:
                         item, valid = validate_data(
                             value=item, item_type_to_return_to=item_matching_validation_model_variable,
-                            expected_value_type=item_matching_validation_model_variable.field_type,
+                            expected_value_type=item_matching_validation_model_variable.field_type
                         )
                         if valid is True:
                             value[key] = item
@@ -181,11 +194,15 @@ def validate_data(value: Any, expected_value_type: Any, item_type_to_return_to:
             indexes_to_pop: List[int] = []
             for i, item in enumerate(value):
                 if item_type_to_return_to.map_model is not None:
-                    item, valid = validate_data(value=item, expected_value_type=item_type_to_return_to.map_model)
+                    item, valid = validate_data(
+                        value=item, expected_value_type=item_type_to_return_to.map_model,
+                    )
                     if valid is False:
                         indexes_to_pop.append(i)
                 elif item_type_to_return_to.items_excepted_type is not None:
-                    item, valid = validate_data(value=item, expected_value_type=item_type_to_return_to.items_excepted_type)
+                    item, valid = validate_data(
+                        value=item, expected_value_type=item_type_to_return_to.items_excepted_type,
+                    )
                     if valid is False:
                         indexes_to_pop.append(i)
                 # If no map validator has been found, this means we have an untyped list. So, we will
@@ -229,20 +246,12 @@ def validate_data(value: Any, expected_value_type: Any, item_type_to_return_to:
                 return value, True
         return value, True
 
+    """
+    # Even tough DynamoDB does not support float types, the conversion 
+    between floats to Decimal is being done in the DynamoDBCore functions
     elif value_type == float:
         # DynamoDB does not support float types. They must be converted to Decimal's.
-        return float_to_decimal(float_number=value), True
+        return value, True
+    """
 
     return value, True
-
-
-def _types_match(type_to_check: type, expected_type: type) -> bool:
-    processed_expected_type: type = try_to_get_primitive_default_type_of_item(expected_type)
-    # The type_to_check will always be a primitive Python type, where as the expected_type can also be a StructNoSQL
-    # model type. In which case, we try_to_get_primitive_default_type_of_item. If the type was not a StructNoSQL
-    # model type and did not had a primitive_default_type, the returned type will be the source type we passed.
-    if processed_expected_type is Any:
-        return True
-    elif type_to_check != processed_expected_type:
-        return False
-    return True
