@@ -3,6 +3,7 @@ from StructNoSQL import TableDataModel, BaseField, MapModel
 
 
 class BaseTableModel(TableDataModel):
+    type = BaseField(field_type=str, required=False)
     fieldOne = BaseField(field_type=str, required=False)
     fieldTwo = BaseField(field_type=str, required=False)
     class ContainerModel(MapModel):
@@ -14,5 +15,5 @@ class BaseTableModel(TableDataModel):
 class DynamoDBTableModel(BaseTableModel):
     accountId = BaseField(field_type=str, required=True)
 
-class InoftVocalEngineTableModel(BaseTableModel):
-    accountProjectUserId = BaseField(field_type=str, required=True)
+class ExternalDynamoDBApiTableModel(BaseTableModel):
+    accountProjectTableKeyId = BaseField(field_type=str, required=True)
